/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/blalloc.h: block allocator header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: blalloc.h,v 1.5 2002/04/08 16:00:45 rageircd Exp $
 */

#ifndef __blalloc_include__
#define __blalloc_include__

#include "setup.h"
#include <stddef.h>

#ifdef HAVE_BLALLOC_ON

typedef struct Block Block;
typedef struct BlockHeap BlockHeap;

struct Block {
	void *elems;
	void *endElem;
	int freeElems;
	unsigned long *allocMap;
	Block *next;
};

struct BlockHeap {
	size_t elemSize;
	int elemsPerBlock;
	int numlongs;
	int blocksAllocated;
	int freeElems;
	Block *base;
};

extern BlockHeap *BlockHeapCreate(size_t, int);
extern int BlockHeapGarbageCollect(BlockHeap *);

#else

typedef struct BlockHeap BlockHeap;
#define BlockHeapCreate(es, epb) ((BlockHeap *)es)
#define BlockHeapGarbageCollect(x)

#endif /* HAVE_BLALLOC_ON */

#endif /* __blalloc_include__ */
