/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/conf2.h: run-time configuration header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: conf2.h,v 1.49 2002/12/16 00:01:42 rageircd Exp $
 */

#ifndef __conf2_include__
#define __conf2_include__ 

#include "setup.h"
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>

typedef struct Network_t Network;			/* network conf */
typedef struct Conf_t Conf;				/* dynamic conf */
typedef struct Auth_t Auth;				/* password authentication */
typedef struct Source_t Source;				/* source userhost */

typedef struct _configfile ConfigFile;			/* configuration files */
typedef struct _configentry ConfigEntry;		/* configuration entry */
typedef struct _configflag ConfigFlag;			/* configuration entry flags */
typedef struct _configitem ConfigItem;			/* generic configuration structure */
typedef struct _configcommand ConfigCommand;		/* configuration commands (parsers) */
typedef struct _configcommandentry ConfigCommandEntry;	/* configuration command entry */
typedef struct _configoption ConfigOption;		/* configuration option */

typedef struct _configitem_servinfo ConfigItem_servinfo; /* servinfo block */
typedef struct _configitem_admin ConfigItem_admin;	/* admin block */
typedef struct _configitem_class ConfigItem_class;	/* class block */
typedef struct _configitem_allow ConfigItem_allow;	/* allow block */
typedef struct _configitem_oper ConfigItem_oper;	/* oper block */
typedef struct _configitem_uline ConfigItem_uline;	/* uline block */
typedef struct _configitem_link ConfigItem_link;	/* link block */
typedef struct _configitem_bind ConfigItem_bind;	/* bind block */
typedef struct _configitem_ban ConfigItem_ban;		/* ban block */
typedef struct _configitem_except ConfigItem_except;	/* except block */
typedef struct _configitem_vhost ConfigItem_vhost;	/* vhost block */
typedef struct _configitem_include ConfigItem_include;	/* include directive */

struct Network_t {
	char *name;
	char *name_005;
	char *klineaddress;
	char *aup_url;
	char *services_server;
	char *stats_server;
	char *netadmin_mask;
	char *admin_mask;
	char *coadmin_mask;
	char *oper_mask;
	char *user_prefix;
};

struct Conf_t {
	Network network;
	int floodcount;
	time_t knockdelay;
	int maxchansperuser;
	int maxclients;
	int linkdepth;
	time_t acceptnoticetime;
	char *klineaddress;
	unsigned int modesonconnect;
	char *autojoinchans;
	time_t connauth_timeout;
#ifdef HAVE_THROTTLE_ON
	int throttle_count;
	time_t throttle_time;
	unsigned throttle_rejected_clients : 1;
#endif
	time_t pacetime_simple;
	time_t pacetime_intense;
	long mask_key1;
	long mask_key2;
	long mask_key3;
	char *proxymonitor_host;
	char *proxymonitor_url;
	unsigned shortmotd : 1;
	unsigned hideulines : 1;
	unsigned checkidentd : 1;
	unsigned showheaders : 1;
	unsigned nomixedcase : 1;
	unsigned failopernotice : 1;
	unsigned ignorefirstchar : 1;
	unsigned logging : 1;
	unsigned dynchans : 1;
	unsigned nooperflood : 1;
	unsigned spynotices : 1;
	unsigned allowknocks : 1;
	time_t statsfile_save_freq;
	time_t anti_spam_quit_message_time;
	unsigned customquitmessages : 1;
	unsigned usernames_in_trace : 1;
	unsigned show_invisible_lusers : 1;
	unsigned anti_nick_flood : 1;
	unsigned anti_away_flood : 1;
	unsigned write_perm_klines_to_file : 1;
	unsigned flatten_links : 1;
	unsigned anti_spambot_checks : 1;
	unsigned anti_acebot_checks : 1;
	unsigned allowmap : 1;
};

struct Auth_t {
	char *string;
	unsigned int type;
};

struct Source_t {
	Source *next, *prev;
	char *userhost;
};

struct _configfile {
	ConfigFile *cf_next;
	char *cf_filename;
	ConfigEntry *cf_entries;
};

struct _configentry {
	ConfigEntry *ce_entries, *ce_next, *ce_prev;
	ConfigFile *ce_fileptr;
	int ce_varlinenum, ce_sectlinenum;
	char *ce_varname;
	char *ce_vardata;
};

struct _configflag {
	unsigned temporary : 1;
	unsigned permanent : 1;
	unsigned removable : 1;
	unsigned remove_on_rehash : 1;
};

struct _configitem {
	ConfigItem *next, *prev;
	ConfigFlag flag;
};

struct _configcommand {
	char *name;
	ConfigItem *(*parse_func)(ConfigEntry *);
	int (*test_func)(ConfigEntry *, ConfigItem *);
	ConfigItem *(*free_func)(ConfigItem *);
};

struct _configcommandentry {
	ConfigCommandEntry *next, *prev;
	ConfigCommand *cc;
	ConfigEntry *ce;
	ConfigItem *ci;
};

struct _configoption {
	char *name;
	unsigned int flag;
};

struct _configitem_servinfo {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *name;
	char *desc;
	char *bindip;
};

struct _configitem_admin {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *line1;
	char *line2;
	char *line3;
};

struct _configitem_class {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *name;
	time_t pingtime;
	time_t connectfreq;
	int maxclients;
	long sendq;
	int clients;
	unsigned int options;
};

struct _configitem_allow {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *hostname;
	char *ip;
	int maxperip;
	ConfigItem_class *class;
	Auth *auth;
	char *redirect_server;
	int redirect_port;
};

struct _configitem_oper {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *name;
	Auth *auth;
	Source *source;
	ConfigItem_class *class;
	unsigned int options;
	char *autojoinchans;
};

struct _configitem_uline {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *servername;
};

struct _configitem_link {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *servername;
	char *fromip;
	char *bindip;
	Auth *auth;
	int port;
	ConfigItem_class *class;
	unsigned int options;
	int refcount;
	struct in_addr ipnum;
	time_t hold;
	unsigned dns_pending : 1;
};

struct _configitem_ban {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	unsigned int type;
	char *mask;
	char *reason;
	time_t expire;
	time_t setat;
};

struct _configitem_except {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *mask;
};

struct _configitem_bind {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *ip;
	int port;
	unsigned int options;
	int clients;
};

struct _configitem_vhost {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *mask;
	char *login;
	Auth *auth;
	Source *source;
};

struct _configitem_include {
	ConfigItem *next, *prev;
	ConfigFlag flag;
	char *filename;
	unsigned loaded : 1;
};

#define IRCNETWORK			dConf.network.name
#define IRCNETWORK005			dConf.network.name_005
#define IRCNETWORK_KLINEADDRESS		dConf.network.klineaddress
#define AUP_URL				dConf.network.aup_url
#define SERVICES_SERVER			dConf.network.services_server
#define STATS_SERVER			dConf.network.stats_server
#define NETADMIN_MASK			dConf.network.netadmin_mask
#define ADMIN_MASK			dConf.network.admin_mask
#define COADMIN_MASK			dConf.network.coadmin_mask
#define OPER_MASK			dConf.network.oper_mask
#define USER_PREFIX			dConf.network.user_prefix
#define MODESONCONNECT			dConf.modesonconnect
#define AUTOJOINCHANS			dConf.autojoinchans
#define CONNAUTH_TIMEOUT		dConf.connauth_timeout
#ifdef HAVE_THROTTLE_ON
#define THROTTLE_COUNT			dConf.throttle_count
#define THROTTLE_TIME			dConf.throttle_time
#define THROTTLE_REJECTED_CLIENTS	dConf.throttle_rejected_clients
#endif
#define PACETIME_SIMPLE			dConf.pacetime_simple
#define PACETIME_INTENSE		dConf.pacetime_intense
#define MASK_KEY1			dConf.mask_key1
#define MASK_KEY2			dConf.mask_key2
#define MASK_KEY3			dConf.mask_key3
#define PROXYMONITOR_HOST		dConf.proxymonitor_host
#define PROXYMONITOR_URL		dConf.proxymonitor_url
#define FLOODCOUNT			dConf.floodcount
#define KNOCKDELAY			dConf.knockdelay
#define MAXCHANSPERUSER			dConf.maxchansperuser
#define MAXCLIENTS			dConf.maxclients
#define LINKDEPTH			dConf.linkdepth
#define ACCEPTNOTICETIME		dConf.acceptnoticetime
#define KLINEADDRESS			dConf.klineaddress
#define SHORTMOTD			dConf.shortmotd
#define HIDEULINES			dConf.hideulines
#define CHECKIDENTD			dConf.checkidentd
#define SHOWHEADERS			dConf.showheaders
#define NOMIXEDCASE			dConf.nomixedcase
#define FAILOPERNOTICE			dConf.failopernotice
#define IGNOREFIRSTCHAR			dConf.ignorefirstchar
#define LOGGING				dConf.logging
#define DYNCHANS			dConf.dynchans
#define NOOPERFLOOD			dConf.nooperflood
#define SPYNOTICES			dConf.spynotices
#define ALLOWKNOCKS			dConf.allowknocks
#define STATSFILE_SAVE_FREQ		dConf.statsfile_save_freq
#define ANTI_SPAM_QUIT_MESSAGE_TIME	dConf.anti_spam_quit_message_time
#define CUSTOMQUITMESSAGES		dConf.customquitmessages
#define USERNAMES_IN_TRACE		dConf.usernames_in_trace
#define SHOW_INVISIBLE_LUSERS		dConf.show_invisible_lusers
#define ANTINICKFLOOD			dConf.anti_nick_flood
#define ANTIAWAYFLOOD			dConf.anti_away_flood
#define WRITE_PERM_KLINES_TO_FILE	dConf.write_perm_klines_to_file
#define FLATTEN_LINKS			dConf.flatten_links
#define ANTI_SPAMBOT_CHECKS		dConf.anti_spambot_checks
#define ANTI_ACEBOT_CHECKS		dConf.anti_acebot_checks
#define ALLOWMAP			dConf.allowmap

#define AUTH_RIPEMD_160			0
#define AUTH_SHA1_160			1
#define AUTH_MD5_128			2
#define AUTH_MD5_56			3
#define AUTH_DES_56			4
#define AUTH_PLAINTEXT			5

#define CLASS_IGNOREIDENTD		0x00000001
#define CLASS_REQUIREIDENTD		0x00000002
#define CLASS_NOTILDE			0x00000004

#define LINK_AUTOCONNECT		0x00000001
#define LINK_COMPRESS			0x00000002
#define LINK_SECURE			0x00000004

#define BIND_BOUND			0x00000001
#define BIND_CLIENTONLY			0x00000002
#define BIND_SERVERONLY			0x00000004
#define BIND_SECURE			0x00000008
#define BIND_CRJAVA			0x00000010

#define BAN_USERHOST			0x00000001
#define BAN_USERHOST_S			0x00000002
#define BAN_IP				0x00000004
#define BAN_IP_S			0x00000008
#define BAN_NICKNAME			0x00000010
#define BAN_NICKNAME_S			0x00000020
#define BAN_GCOS			0x00000040
#define BAN_GCOS_S			0x00000080
#define BAN_SERVER			0x00000100
#define BAN_FILE			0x00000200
#define BAN_CHANNEL			0x00000400

#define ConfDupString(x, y)	if ((x) != NULL) { MyFree((x)); } DupString(x, y)
#define ConfFree(x)		if ((x) != NULL) { MyFree((x)); } (x) = NULL

#define CheckBlockName(x, y, z)	if (BadPtr((x)->ce_vardata)) { report(0, "%s:%d: %s: missing %s", \
					(x)->ce_fileptr->cf_filename, (x)->ce_varlinenum, (y), (z)); return NULL; }
#define CheckBlockEntries(x, y)	if ((x)->ce_entries == NULL) { report(0, "%s:%d: %s: empty block", \
					(x)->ce_fileptr->cf_filename, (x)->ce_varlinenum, (y)); return NULL; }
#define CheckSection(x, y)	if ((x)->ce_entries == NULL) { report(0, "%s:%d: %s: empty section", \
					(x)->ce_fileptr->cf_filename, (x)->ce_varlinenum, (y)); continue; }
#define CheckVar(x, y)		if (BadPtr((x)->ce_varname)) { report(0, "%s:%d: %s: missing variable name", \
					(x)->ce_fileptr->cf_filename, (x)->ce_varlinenum, (y)); continue; }
#define CheckParam(x, y)	if (BadPtr((x)->ce_vardata)) { report(0, "%s:%d: %s: missing variable parameter", \
					(x)->ce_fileptr->cf_filename, (x)->ce_varlinenum, (y)); continue; }
#define UnknownVar(x, y)	report(0, "%s:%d: %s: unknown variable %s", (x)->ce_fileptr->cf_filename, \
					(x)->ce_varlinenum, (y), (x)->ce_varname);
#define UnknownClass(x, y)	report(0, "%s:%d: %s: unknown class %s (using default)", \
					(x)->ce_fileptr->cf_filename, (x)->ce_varlinenum, (y), (x)->ce_vardata)

#endif /* __conf2_include__ */
