/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/h.h: prototypes header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: h.h,v 1.80 2002/12/22 18:11:44 rageircd Exp $
 */

#include "send.h"
#include "ircsprintf.h"
#include "fdlist.h"
#include "setup.h"

/* Configuration */
extern ConfigItem_servinfo *ci_servinfo;
extern ConfigItem_admin *ci_admin;
extern ConfigItem_class *ci_class;
extern ConfigItem_allow *ci_allow;
extern ConfigItem_oper *ci_oper;
extern ConfigItem_uline *ci_uline;
extern ConfigItem_link *ci_link;
extern ConfigItem_bind *ci_bind;
extern ConfigItem_ban *ci_ban;
extern ConfigItem_except *ci_except;
extern ConfigItem_vhost *ci_vhost;

extern ConfigItem_class *default_class;
extern Conf dConf;

extern void rehash(int);
extern void add_ConfigItem(ConfigItem *, ConfigItem **);
extern ConfigItem *del_ConfigItem(ConfigItem *, ConfigItem **);
extern void report(short, char *, ...);
extern int conf_doban(aClient *, aClient *, ConfigItem_ban *, char *, int);
extern ConfigItem_class *find_class(char *);
extern ConfigItem_oper *find_oper(char *);
extern ConfigItem_uline *find_uline(char *);
extern ConfigItem_link *find_link(char *, char *);
extern ConfigItem_bind *find_bind(char *, int);
extern ConfigItem_ban *find_ban(long, char *);
extern ConfigItem_except *find_except(char *);

#ifndef STATIC_MODULES
extern void init_Modules();
extern void check_Modules();
extern void handle_module_call(char *, aClient *, int);
extern void load_modules_from_path(char *);
extern void unload_all_modules();
#endif
extern ModuleR RequiredModules;

extern int do_bancheck;
extern int spam_num, spam_time;

extern int drone_check_spambot(aClient *, char);
extern int drone_check_acebot(aClient *);

extern int base64_encode(const char *, size_t, char *, size_t);
extern int base64_decode(const char *, char *, size_t);

extern ConnectAuth *connauth_list;
extern void connauth_timeout_queries(time_t);
extern void connauth_init(aClient *);
extern void connauth_send_id(ConnectAuth *);
extern void connauth_read_id(ConnectAuth *);

extern char *crypt();
extern char *maskme(aClient *);
extern void mask_client(aClient *, char *);

extern Auth *auth_parse_conf_block(ConfigEntry *, char *);
extern int auth_check(Auth *, char *);
extern void auth_destroy_struct(Auth *);
extern void auth_handle_mkpasswd(aClient *, char *, char *);
extern void *source_parse_conf_block(ConfigEntry *, char *, Source **);
extern Source *source_del(Source *, Source **);
extern void source_destroy_list(Source **);
extern int source_check(Source **, char *);

extern int msg_has_colour(char *);
extern unsigned char *strip_colour(char *);

extern Listener *listener_list;
extern int listener_count;
extern int add_listener(ConfigItem_bind *);
extern void close_listeners();
extern char *get_listener_name(Listener *);
extern Listener *find_listener(char *, int);

extern MessageFile motd;
extern MessageFile opermotd;
extern MessageFile rules;
extern void load_message_file(MessageFile *);
extern void send_message_file(aClient *, MessageFile *);

extern int lifesux;
extern int currlife;
extern int LCF;
extern int LRV;
extern int noisy_htm;
extern int HTMLOCK;

extern fdlist serv_fdlist;
extern fdlist busycli_fdlist;
extern fdlist default_fdlist;
extern fdlist oper_fdlist;
extern fdlist listen_fdlist;
extern struct Counter Count;

extern time_t NOW;
extern time_t last_stat_save;
extern time_t nextconnect, nextping, nextevent, timeofday;
extern aClient *client, me, *local[];
extern aChannel *channel;
extern struct stats *ircstp;

extern char *canonize(char *);
extern void check_fdlists();
extern aChannel *find_channel(char *, aChannel *);
extern int is_banned(aClient *, aChannel *);
extern int has_invex(aClient *, aChannel *);
extern chanMember *remove_user_from_channel(aClient *, aChannel *);
extern void send_user_joins(aClient *, aClient *);
extern int can_send(aClient *, aChannel *, char *);
extern int is_chan_op(aClient *, aChannel *);
extern int is_half_op(aClient *, aChannel *);
extern int is_chan_admin(aClient *, aChannel *);
extern int has_voice(aClient *, aChannel *);
extern int count_channels(aClient *);
extern char *pretty_mask(char *);
extern int check_channelname(aClient *, unsigned char *);
extern void add_invite(aClient *, aChannel *);
extern void del_invite(aClient *, aChannel *);
extern int add_ban_tolist(aClient *, aChannel *, char *, aBan **, int);
extern int del_ban_fromlist(aChannel *, char *, aBan **, int);
extern void channel_modes(aClient *, char *, char *, aChannel *);
extern aClient *find_chasing(aClient *, char *, int *);

extern void clear_scache_hash_table(void);
extern char *find_or_add(char *);
extern void count_scache(int *, u_long *);
extern void list_scache(aClient *, aClient *, int, char **);

extern char *my_itoa(int);

extern aCommand *CommandHash[256];
extern void init_CommandHash();
extern void addCommand(char *, int (*)(), int, int);
extern void delCommand(char *, int (*)());

extern Hook *HookTable[MAXHOOKTYPES];
extern Hook *ghook;
extern void init_HookTable();
extern void addHook(int, int (*)(), void (*)());
extern void delHook(int, int (*)(), void (*)());

extern char *rpl_str(int);
extern char *err_str(int);
extern char *getreply(int);
extern char *strerror(int);
extern int dgets(int, char *, int);
extern char *inetntoa(char *);
extern char *debug_mode;
extern int debug_level;
extern int errno, h_errno;
extern int highest_fd;
extern int readcalls;
extern int check_client(aClient *);
extern void close_connection(aClient *);
extern int connect_server(ConfigItem_link *, aClient *, DNSQuery *);
extern void get_my_name(aClient *, char *, int);
extern int get_sockerr(aClient *);
extern int inetport(aClient *, char *, int);
extern int read_message(time_t, fdlist *);
extern void report_error(char *, aClient *);
extern void set_non_blocking(int);
extern int set_blocking(int);

extern void restart(char *);
extern void send_channel_modes(aClient *, aChannel *);
extern void server_reboot(void);

extern int match(char *, char *);
extern char *collapse(char *);

extern int writecalls, writeb[];
extern int ircsend(aClient *, const void *, size_t);
extern int ircrecv(aClient *, void *, size_t);
extern int deliver_it(aClient *, char *, int);

extern char *check_string(char *);
extern char *make_nick_user_host(char *, char *, char *);
extern char *make_user_host(char *, char *);
extern char *get_client_name(aClient *, int);
extern char *myctime(time_t), *date(time_t);
extern int exit_client(aClient *, aClient *, aClient *, char *);
extern void init_stats(void), tstats(aClient *, char *);
extern void serv_info(aClient *, char *);

extern int parse(aClient *, char *, char *);

extern int do_numeric(int, aClient *, aClient *, int, char **);
extern int hunt_server(aClient *, aClient *, char *, int, int, char **);
extern aClient *next_client(aClient *, char *);
extern aClient *next_client_double(aClient *, char *);

extern int send_lusers(aClient *, aClient *, int, char **);
extern void send_umode(aClient *, aClient *, int, int, char *);
extern void send_umode_out(aClient *, aClient *, int);
extern int del_silence(aClient *, char *);

extern aChannel *make_channel();
extern aBan *make_channelban();
extern Link *make_slink(void);
extern chanMember *make_chanmember(void);
extern anUser *make_user(aClient *);
extern aServer *make_server(aClient *);
extern aClient *make_client(aClient *, aClient *);
#ifdef HAVE_THROTTLE_ON
extern throttle *make_throttle();
extern hashent *make_hashent();
extern void free_throttle(throttle *);
extern void free_hashent(hashent *);
#endif
extern void free_client(aClient *);
extern void free_slink(Link *);
extern void free_chanmember(chanMember *);
extern void free_user(anUser *, aClient *);
extern void free_channel(aChannel *);
extern void free_channelban(aBan *);
extern chanMember *find_user_member(chanMember *, aChannel *);
extern Link *find_str_link(Link *, char *);
extern void add_client_to_list(aClient *);
extern void remove_client_from_list(aClient *);

extern void clear_channel_hash_table(void);
extern void clear_client_hash_table(void);
extern int add_to_client_hash_table(char *, aClient *);
extern int del_from_client_hash_table(char *, aClient *);
extern int add_to_channel_hash_table(char *, aChannel *);
extern int del_from_channel_hash_table(char *, aChannel *);
extern aChannel *hash_find_channel(char *, aChannel *);
extern aClient *hash_find_client(char *, aClient *);
extern aClient *hash_find_nickserver(char *, aClient *);
extern aClient *hash_find_server(char *, aClient *);

extern void add_history(aClient *, int);
extern aClient *get_history(char *, time_t);
extern void off_history(aClient *);
extern void init_whowas();

extern int dopacket(aClient *, char *, int);
extern int client_dopacket(aClient *, char *, int);

extern void debug(int level, char *pattern, ...);
extern void ircdlog(char *, char *, ...);

#ifdef DEBUGMODE
extern void send_usage(aClient *, char *);
extern void send_listinfo(aClient *, char *);
extern void count_memory(aClient *, char *);
#endif

extern void send_list(aClient *, int);
extern aChannel *hash_get_chan_bucket(int);

extern void remove_one_ip(unsigned long);
extern void clear_ip_hash_table(void);
extern void rehash_ip_hash(void);

extern u_long inet_addr(char *);

#ifdef FLUD
int check_for_flood();
void free_fluders();
void free_fludees();
#define MyFludConnect(x)	(((x)->fd >= 0) || ((x)->fd == -2))
#endif

extern void addEvent(const char *, void (*)(), void *, time_t, unsigned);
extern void delEvent(void (*)(), void *);
extern void checkEvent(void);
extern time_t next_checkEvent(void);
extern void show_events(aClient *);

extern void count_watch_memory(int *, u_long *);
extern void clear_watch_hash_table(void);
extern int add_to_watch_hash_table(char *, aClient *);
extern int del_from_watch_hash_table(char *, aClient *);
extern int hash_check_watch(aClient *, int);
extern int hash_del_watch_list(aClient  *);
extern aWatch *hash_get_watch(char *);

extern time_t atime(char *);
extern char *secstodhms(time_t);
extern void strip(char *);

#include "find.h"
