/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/commands.c: commands module wrapper
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: commands.c,v 1.15 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module _module_header = {
	"modules/commands.so",
	"Commands Module",
	4,
	"$Revision: 1.15 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("AWAY",		m_away,		MAXPARA,	MSG_RESETIDLE);
	addCommand("MAP",		m_map,		MAXPARA,	MSG_NONE);
	addCommand("MKPASSWD",		m_mkpasswd,	MAXPARA,	MSG_NONE);
	addCommand("OPER",		m_oper,		MAXPARA,	MSG_NONE);
	addCommand("VERSION",		m_version,	MAXPARA,	MSG_UNREGUSE);
	addCommand("SVSMODE",		m_svsmode,	MAXPARA,	MSG_NONE);
	addCommand("SVSNICK",		m_svsnick,	MAXPARA,	MSG_NONE);
	addCommand("JOIN",		m_join,		MAXPARA,	MSG_RESETIDLE);
	addCommand("PART",		m_part,		MAXPARA,	MSG_RESETIDLE);
	addCommand("KLINE",		m_kline,	MAXPARA,	MSG_NONE);
	addCommand("UNKLINE",		m_unkline,	MAXPARA,	MSG_NONE);
	addCommand("DIE",		m_die,		MAXPARA,	MSG_NONE);
	addCommand("RESTART",		m_restart,	MAXPARA,	MSG_NONE);
	addCommand("ZLINE",		m_zline,	MAXPARA,	MSG_NONE);
	addCommand("SQLINE",		m_sqline,	MAXPARA,	MSG_NONE);
	addCommand("UNSQLINE",		m_unsqline,	MAXPARA,	MSG_NONE);
	addCommand("WHOIS",		m_whois,	MAXPARA,	MSG_NONE);
	addCommand("WHOWAS",		m_whowas,	MAXPARA,	MSG_NONE);
	addCommand("ADMIN",		m_admin,	MAXPARA,	MSG_UNREGUSE);
	addCommand("INFO",		m_info,		MAXPARA,	MSG_NONE);
	addCommand("DALINFO",		m_dalinfo,	MAXPARA,	MSG_NONE);
	addCommand("LINKS",		m_links,	MAXPARA,	MSG_NONE);
	addCommand("KICK",		m_kick,		MAXPARA,	MSG_NONE);
	addCommand("VHOST",		m_vhost,	MAXPARA,	MSG_NONE);
	addCommand("MODE",		m_mode,		MAXPARA,	MSG_NONE);
	addCommand("TOPIC",		m_topic,	MAXPARA,	MSG_NONE);
	addCommand("INVITE",		m_invite,	MAXPARA,	MSG_NONE);
	addCommand("NAMES",		m_names,	MAXPARA,	MSG_NONE);
	addCommand("TIME",		m_time,		MAXPARA,	MSG_NONE);
	addCommand("PING",		m_ping,		MAXPARA,	MSG_NONE);
	addCommand("PONG",		m_pong,		MAXPARA,	MSG_NONE);
	addCommand("QUIT",		m_quit,		MAXPARA,	MSG_NONE);
	addCommand("PASS",		m_pass,		MAXPARA,	MSG_UNREGUSE);
	addCommand("USERHOST",		m_userhost,	MAXPARA,	MSG_NONE);
	addCommand("ISON",		m_ison,		MAXPARA,	MSG_NONE);
	addCommand("MOTD",		m_motd,		MAXPARA,	MSG_NONE);
	addCommand("RULES",		m_rules,	MAXPARA,	MSG_NONE);
	addCommand("OPERMOTD",		m_opermotd,	MAXPARA,	MSG_NONE);
	addCommand("SVSNOOP",		m_svsnoop,	MAXPARA,	MSG_NONE);
	addCommand("LIST",		m_list,		MAXPARA,	MSG_NONE);
	addCommand("SAMODE",		m_samode,	MAXPARA,	MSG_NONE);
	addCommand("CAPAB",		m_capab,	MAXPARA,	MSG_UNREGUSE);
	addCommand("SJOIN",		m_sjoin,	MAXPARA,	MSG_NONE);
	addCommand("REHASH",		m_rehash,	MAXPARA,	MSG_NONE);
	addCommand("HTM",		m_htm,		MAXPARA,	MSG_NONE);
	addCommand("BURST",		m_burst,	MAXPARA,	MSG_NONE);
	addCommand("SVINFO",		m_svinfo,	MAXPARA,	MSG_UNREGUSE);
	addCommand("USERS",		m_users,	MAXPARA,	MSG_NONE);
	addCommand("ERROR",		m_error,	MAXPARA,	MSG_UNREGUSE);
	addCommand("LUSERS",		m_lusers,	MAXPARA,	MSG_NONE);
	addCommand("CONNECT",		m_connect,	MAXPARA,	MSG_NONE);
	addCommand("WALLOPS",		m_wallops,	MAXPARA,	MSG_NONE);
	addCommand("LOCOPS",		m_locops,	MAXPARA,	MSG_NONE);
	addCommand("GLOBOPS",		m_globops,	MAXPARA,	MSG_NONE);
	addCommand("GNOTICE",		m_gnotice,	MAXPARA,	MSG_NONE);
	addCommand("SQUIT",		m_squit,	MAXPARA,	MSG_NONE);
	addCommand("CHATOPS",		m_chatops,	MAXPARA,	MSG_NONE);
	addCommand("CLOSE",		m_close,	MAXPARA,	MSG_NONE);
	addCommand("NICK",		m_nick,		MAXPARA,	MSG_UNREGUSE);
	addCommand("USER",		m_user,		MAXPARA,	MSG_UNREGUSE);
	addCommand("PRIVMSG",		m_private,	MAXPARA,	MSG_RESETIDLE);
	addCommand("NOTICE",		m_notice,	MAXPARA,	MSG_NONE);
	addCommand("KILL",		m_kill,		MAXPARA,	MSG_NONE);
	addCommand("SVSKILL",		m_svskill,	MAXPARA,	MSG_NONE);
	addCommand("AKILL",		m_akill,	MAXPARA,	MSG_NONE);
	addCommand("RAKILL",		m_rakill,	MAXPARA,	MSG_NONE);
	addCommand("DKEY",		m_dkey,		MAXPARA,	MSG_UNREGUSE);
	addCommand("TRACE",		m_trace,	MAXPARA,	MSG_NONE);
	addCommand("WATCH",		m_watch,	MAXPARA,	MSG_NONE);
	addCommand("DCCALLOW",		m_dccallow,	MAXPARA,	MSG_NONE);
	addCommand("SILENCE",		m_silence,	MAXPARA,	MSG_NONE);
	addCommand("STATS",		m_stats,	MAXPARA,	MSG_NONE);
	addCommand("WHO",		m_who,		MAXPARA,	MSG_NONE);
	addCommand("SERVER",		m_server,	MAXPARA,	MSG_UNREGUSE);
	addCommand("KNOCK",		m_knock,	MAXPARA,	MSG_NONE);
	addCommand("SET",		m_set,		MAXPARA,	MSG_NONE);
	addCommand("SGLINE",		m_sgline,	MAXPARA,	MSG_NONE);
	addCommand("UNSGLINE",		m_unsgline,	MAXPARA,	MSG_NONE);
	addCommand("ACCEPT",		m_accept,	MAXPARA,	MSG_NONE);
	addCommand("SAJOIN",		m_sajoin,	MAXPARA,	MSG_NONE);
	addCommand("SAKICK",		m_sakick,	MAXPARA,	MSG_NONE);
	addCommand("SZLINE",		m_szline,	MAXPARA,	MSG_NONE);
	addCommand("UNSZLINE",		m_unszline,	MAXPARA,	MSG_NONE);
	addCommand("SNICK",		m_snick,	MAXPARA,	MSG_NONE);
	addHook(HOOKTYPE_JOIN, m_join, NULL);
	RequiredModules.commands = 1;
	return;
}
void _module_unload() {
	delCommand("AWAY",		m_away);
	delCommand("MAP",		m_map);
	delCommand("MKPASSWD",		m_mkpasswd);
	delCommand("OPER",		m_oper);
	delCommand("VERSION",		m_version);
	delCommand("SVSMODE",		m_svsmode);
	delCommand("SVSNICK",		m_svsnick);
	delCommand("JOIN",		m_join);
	delCommand("PART",		m_part);
	delCommand("KLINE",		m_kline);
	delCommand("UNKLINE",		m_unkline);
	delCommand("DIE",		m_die);
	delCommand("RESTART",		m_restart);
	delCommand("ZLINE",		m_zline);
	delCommand("SQLINE",		m_sqline);
	delCommand("UNSQLINE",		m_unsqline);
	delCommand("WHOIS",		m_whois);
	delCommand("WHOWAS",		m_whowas);
	delCommand("ADMIN",		m_admin);
	delCommand("INFO",		m_info);
	delCommand("DALINFO",		m_dalinfo);
	delCommand("LINKS",		m_links);
	delCommand("KICK",		m_kick);
	delCommand("VHOST",		m_vhost);
	delCommand("MODE",		m_mode);
	delCommand("TOPIC",		m_topic);
	delCommand("INVITE",		m_invite);
	delCommand("NAMES",		m_names);
	delCommand("TIME",		m_time);
	delCommand("PING",		m_ping);
	delCommand("PONG",		m_pong);
	delCommand("QUIT",		m_quit);
	delCommand("PASS",		m_pass);
	delCommand("USERHOST",		m_userhost);
	delCommand("ISON",		m_ison);
	delCommand("MOTD",		m_motd);
	delCommand("RULES",		m_rules);
	delCommand("OPERMOTD",		m_opermotd);
	delCommand("SVSNOOP",		m_svsnoop);
	delCommand("LIST",		m_list);
	delCommand("SAMODE",		m_samode);
	delCommand("CAPAB",		m_capab);
	delCommand("SJOIN",		m_sjoin);
	delCommand("REHASH",		m_rehash);
	delCommand("HTM",		m_htm);
	delCommand("BURST",		m_burst);
	delCommand("SVINFO",		m_svinfo);
	delCommand("USERS",		m_users);
	delCommand("ERROR",		m_error);
	delCommand("LUSERS",		m_lusers);
	delCommand("CONNECT",		m_connect);
	delCommand("WALLOPS",		m_wallops);
	delCommand("LOCOPS",		m_locops);
	delCommand("GLOBOPS",		m_globops);
	delCommand("GNOTICE",		m_gnotice);
	delCommand("SQUIT",		m_squit);
	delCommand("CHATOPS",		m_chatops);
	delCommand("CLOSE",		m_close);
	delCommand("NICK",		m_nick);
	delCommand("USER",		m_user);
	delCommand("PRIVMSG",		m_private);
	delCommand("NOTICE",		m_notice);
	delCommand("KILL",		m_kill);
	delCommand("SVSKILL",		m_svskill);
	delCommand("AKILL",		m_akill);
	delCommand("RAKILL",		m_rakill);
	delCommand("DKEY",		m_dkey);
	delCommand("TRACE",		m_trace);
	delCommand("WATCH",		m_watch);
	delCommand("DCCALLOW",		m_dccallow);
	delCommand("SILENCE",		m_silence);
	delCommand("STATS",		m_stats);
	delCommand("WHO",		m_who);
	delCommand("SERVER",		m_server);
	delCommand("KNOCK",		m_knock);
	delCommand("SET",		m_set);
	delCommand("SGLINE",		m_sgline);
	delCommand("UNSGLINE",		m_unsgline);
	delCommand("ACCEPT",		m_accept);
	delCommand("SAJOIN",		m_sajoin);
	delCommand("SAKICK",		m_sakick);
	delCommand("SZLINE",		m_szline);
	delCommand("UNSZLINE",		m_unszline);
	delCommand("SNICK",		m_snick);
	delHook(HOOKTYPE_JOIN, m_join, NULL);
	RequiredModules.commands = 0;
	return;
}
