/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_akill.c: AKILL command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_akill.c,v 1.17 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern void send_globops(char *, ...);
extern char *smalldate(time_t);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_akill.so",
	"/AKILL command",
	4,
	"$Revision: 1.17 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("AKILL", m_akill, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("AKILL", m_akill);
	return;
}
#endif

/*
 * m_akill
 *	parv[0] = sender prefix
 *	parv[1] = host
 *	parv[2] = user
 *	parv[3] = length
 *	parv[4] = akiller
 *	parv[5] = time set
 *	parv[6] = reason
 */
int m_akill(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	ConfigItem_ban *ban;
	char *user, *host, *userhost, *akiller, *current_time, *reason, buffer[1024];
	time_t length = 0, timeset = 0;

	if (!IsServer(sptr) || parc < 6) {
		return 0;
	}
	if (!IsULine(sptr)) {
		send_globops("Non-ULined server %s is tring to AKILL!", parv[0]);
		sendto_serv_butone(&me, ":%s GLOBOPS :Non-ULined server %s is trying to AKILL!",
			me.name, parv[0]);
		return 0;
	}

	user = parv[2];
	host = parv[1];
	akiller = parv[4];
	length = atoi(parv[3]);
	timeset = atoi(parv[5]);
	reason = !BadPtr(parv[6]) ? parv[6] : "no reason";
	current_time = smalldate((time_t)timeset);

	if (strlen(reason) > 250) {
		reason[250] = '\0';
	}

	userhost = make_user_host(user, host);
	if ((ban = find_ban(BAN_USERHOST_S, userhost)) == NULL) {
		ban = (ConfigItem_ban *)MyMallocX(sizeof(ConfigItem_ban));
		ban->type = (BAN_USERHOST|BAN_USERHOST_S);
		DupString(ban->mask, userhost);
		ircsprintf(buffer, "%s (%s)", reason, current_time);
		DupString(ban->reason, buffer);
		add_ConfigItem((ConfigItem *)ban, (ConfigItem **)&ci_ban);
		ircdlog(LOG_BAN, "AUTOKILL on %s (%s) added by %s", ban->mask, ban->reason,
			get_client_name(sptr, TRUE));
	}
	sendto_serv_butone(cptr, ":%s AKILL %s %s %d %s %d :%s", sptr->name, host, user, length,
		akiller, timeset, reason);
	do_bancheck = 1;
	return 0;
}
