/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_burst.c: BURST protocol module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_burst.c,v 1.12 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_burst.so",
	"BURST protocol",
	4,
	"$Revision: 1.12 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("BURST", m_burst, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("BURST", m_burst);
	return;
}
#endif

/*
 * m_burst
 *	parv[0] = sender prefix
 *	parv[1] = SendQ if an EOB
 */
int m_burst(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!IsServer(sptr) || sptr != cptr || parc > 2 || !HasCapab(sptr, CAPAB_BURST)) {
		return 0;
	}
	if (parc == 2) {
		sptr->serv->sflags &= ~SFLAGS_EOBRECV;
		if (sptr->serv->sflags & (SFLAGS_SOBSENT|SFLAGS_BURST)) {
			return 0;
		}
#ifdef HTM_LOCK_ON_NETBURST
		HTMLOCK = 0;
#endif
		sendto_gnotice("synch to %s in %d %s at %s sendq", *parv, timeofday - sptr->firsttime,
			timeofday - sptr->firsttime == 1 ? "sec" : "secs", parv[1]);
		sendto_serv_butone(NULL, ":%s GNOTICE :synch to %s in %d %s at %s sendq", me.name,
			*parv, timeofday - sptr->firsttime,
			timeofday - sptr->firsttime == 1 ? "sec" : "secs", parv[1]);
	}
	else {
#ifdef HTM_LOCK_ON_NETBURST
		NTMLOCK = 1;
#endif
		sptr->serv->sflags |= SFLAGS_EOBRECV;
	}
	return 0;
}
