/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_gnotice.c: GNOTICE protocol module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_gnotice.c,v 1.8 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_gnotice.so",
	"GNOTICE protocol",
	4,
	"$Revision: 1.8 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("GNOTICE", m_gnotice, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("GNOTICE", m_gnotice);
	return;
}
#endif

/*
 * m_gnotice
 *	parv[0] = sender prefix
 *	parv[1] = message
 */
int m_gnotice(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!IsServer(sptr) && MyConnect(sptr)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		if (MyClient(sptr)) {
			sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "GNOTICE");
		}
		return 0;
	}
	sendto_serv_butone(IsServer(cptr) ? cptr : NULL, ":%s GNOTICE :%s", parv[0], parv[1]);
	sendto_gnotice("from %s: %s", parv[0], parv[1]);
	return 0;
}
