/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_htm.c: HTM command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_htm.c,v 1.10 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_htm.so",
	"/HTM command",
	4,
	"$Revision: 1.10 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("HTM", m_htm, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("HTM", m_htm);
	return;
}
#endif

/*
 * m_htm
 *	parv[0] = sender prefix
 *	parv[1] = optional command
 *	parv[2] = optional value
 */
int m_htm(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int new_value = 0;

	if (!MyClient(sptr) || !HasMode(sptr, UMODE_OPER)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}
	if (parc < 2) {
		sendto_one(sptr, ":%s NOTICE %s :HTM is %s (%d), %s. Max rate is %dk/s. Current is %dk/s.",
			me.name, parv[0], lifesux ? "ON" : "OFF", lifesux, noisy_htm == 1 ? "NOISY" : "QUIET",
			LRV, currlife);
		return 0;
	}
	if (parc > 1) {
		if (!mycmp(parv[1], "TO")) {
			if (parc < 3) {
				sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "HTM TO");
				return 0;
			}
			if ((new_value = atoi(parv[2])) < 10) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set LRV < 10.",
					me.name, parv[0]);
			}
			else {
				LRV = new_value;
			}
			sendto_one(sptr, ":%s NOTICE %s :New max rate is %dk/s. Current is %dk/s.", me.name,
				parv[0], LRV, currlife);
			sendto_realops("%s (%s@%s) set HTM rate to %dk/s. Current is %dk/s.", parv[0],
				sptr->username, MaskedHost(sptr), LRV, currlife);
		}
		else if (!mycmp(parv[1], "ON")) {
			lifesux = 1;
			sendto_one(sptr, ":%s NOTICE %s :HTM is now ON", me.name, parv[0]);
			sendto_realops("Entering high traffic mode (HTM): forced by %s (%s@%s)", parv[0],
				sptr->username, MaskedHost(sptr));
			LCF = 30;
		}
		else if (!mycmp(parv[1], "OFF")) {
#ifdef HTM_LOCK_ON_NETBURST
			if (HTMLOCK == YES) {
				if (!HasMode(sptr, UMODE_NETADMIN)) {
					sendto_one(sptr, ":%s NOTICE %s :Cannot change HTM: currently LOCKED.",
						me.name, parv[0]);
				}
				else {
					sendto_one(sptr, ":%s NOTICE %s :Removing HTM LOCK", me.name, parv[0]);
					sendto_ops("Resuming standard operation through HTM LOCK: forced by "
						"%s (%s@%s)", parv[0], sptr->username, MaskedHost(sptr));
					lifesux = 0;
					LCF = LOADCFREQ;
				}
				return 0;
			}
#else
			lifesux = 0;
			LCF = LOADCFREQ;
			sendto_one(sptr, ":%s NOTICE %s :HTM is now OFF", me.name, parv[0]);
			sendto_ops("Resuming standard operation: forced by %s (%s@%s)", parv[0],
				sptr->username, MaskedHost(sptr));
#endif
		}
		else if (!mycmp(parv[1], "NOISY")) {
			sendto_ops("HTM is now NOISY");
			noisy_htm = 1;
		}
		else if (!mycmp(parv[1], "QUIET")) {
			sendto_ops("HTM is now QUIET");
			noisy_htm = 0;
		}
		else {
			sendto_one(sptr, ":%s NOTICE %s :Syntax: /HTM <TO/ON/OFF/NOISY/QUIET> [value]",
				me.name, parv[0]);
		}
	}
	return 0;
}
