/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_invite.c: INVITE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_invite.c,v 1.10 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_invite.so",
	"/INVITE command",
	4,
	"$Revision: 1.10 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("INVITE", m_invite, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("INVITE", m_invite);
	return;
}
#endif

/*
 * m_invite
 *	parv[0] = sender prefix
 *	parv[1] = user to invite
 *	parv[2] = channel
 */
int m_invite(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	aChannel *chptr;

	if (parc < 3 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "INVITE");
		return 0;
	}

	if ((acptr = find_person(parv[1], NULL)) == NULL) {
		sendto_one(sptr, err_str(ERR_NOSUCHNICK), me.name, parv[0], parv[1]);
		return 0;
	}
	if (!check_channelname(sptr, (unsigned char *)parv[2])) {
		return 0;
	}
	if ((chptr = find_channel(parv[2], NULL)) == NULL) {
		sendto_prefix_one(acptr, sptr, ":%s INVITE %s :%s", parv[0], parv[1],
			parv[2]);
		return 0;
	}
	if (!IsMember(sptr, chptr) && !IsULine(sptr)) {
		sendto_one(sptr, err_str(ERR_NOTONCHANNEL), me.name, parv[0], parv[2]);
		return 0;
	}
	if (IsMember(acptr, chptr)) {
		sendto_one(sptr, err_str(ERR_USERONCHANNEL), me.name, parv[0], parv[1],
			parv[2]);
		return 0;
	}
	if (chptr->mode.mode & MODE_INVITEONLY) {
		if (!is_chan_op(sptr, chptr) && !is_half_op(sptr, chptr) && !IsULine(sptr)) {
			sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED), me.name, parv[0],
				chptr->chname);
			return 0;
		}
		else if (!IsMember(sptr, chptr) && !IsULine(sptr)) {
			sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED), me.name, parv[0],
				chptr ? chptr->chname : parv[2]);
			return 0;
		}
	}
	if (MyConnect(sptr)) {
		sendto_one(sptr, rpl_str(RPL_INVITING), me.name, parv[0], acptr->name,
			chptr ? chptr->chname : parv[2]);
		if (acptr->user->away != NULL) {
			sendto_one(sptr, rpl_str(RPL_AWAY), me.name, parv[0], acptr->name,
				acptr->user->away);
		}
	}
	if (MyConnect(acptr)) {
		if ((sptr->user != NULL && is_chan_op(sptr, chptr)) || IsULine(sptr)) {
			add_invite(acptr, chptr);
			sendto_channelflag_butone(CHFL_CHANOP, NULL, &me, chptr, ":%s NOTICE @%s "
				":%s invited %s into channel %s", me.name, chptr->chname, sptr->name,
				acptr->name, chptr->chname);
		}
	}
	sendto_prefix_one(acptr, sptr, ":%s INVITE %s :%s", parv[0], acptr->name,
		chptr ? chptr->chname : parv[2]);
	return 0;
}
