/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_mkpasswd.c: MKPASSWD command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_mkpasswd.c,v 1.16 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_mkpasswd.so",
	"/MKPASSWD command",
	4,
	"$Revision: 1.16 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("MKPASSWD", m_mkpasswd, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("MKPASSWD", m_mkpasswd);
	return;
}
#endif

/*
 * m_mkpasswd
 *	parv[0] = sender prefix
 *	parv[1] = type
 *	parv[2] = string
 */
int m_mkpasswd(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!HasMode(sptr, UMODE_OPER)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0], "MKPASSWD");
		return 0;
	}
	if (parc < 3 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "MKPASSWD");
		return 0;
	}
	auth_handle_mkpasswd(sptr, parv[1], parv[2]);
	return 0;
}
