/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_ping.c: PING command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_ping.c,v 1.8 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_ping.so",
	"/PING command",
	4,
	"$Revision: 1.8 $",
	1, NULL, NULL
};
void _module_load() {
	addCommand("PING", m_ping, MAXPARA, MSG_NONE);
	RequiredModules.m_ping = 1;
	return;
}
void _module_unload() {
	delCommand("PING", m_ping);
	RequiredModules.m_ping = 0;
	return;
}
#endif

/*
 * m_ping
 *	parv[0] = sender prefix
 *	parv[1] = origin
 *	parv[2] = destination
 */
int m_ping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *origin, *destination;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NOORIGIN), me.name, parv[0]);
		return 0;
	}

	origin = parv[1];
	destination = parv[2];

	if ((acptr = find_client(origin, NULL)) == NULL) {
		acptr = find_server(origin, NULL);
	}
	if (acptr != NULL && acptr != sptr) {
		origin = cptr->name;
	}
	if (!BadPtr(destination) && mycmp(destination, me.name)) {
		if ((acptr = find_server(destination, NULL)) == NULL) {
			sendto_one(sptr, err_str(ERR_NOSUCHSERVER), me.name, parv[0],
				destination);
			return 0;
		}
		sendto_one(acptr, ":%s PING %s :%s", parv[0], origin, destination);
	}
	else {
		sendto_one(sptr, ":%s PONG %s :%s", me.name, destination ? destination : me.name,
			origin);
	}
	return 0;
}
