/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_quit.c: QUIT command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_quit.c,v 1.11 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_quit.so",
	"/QUIT command",
	4,
	"$Revision: 1.11 $",
	1, NULL, NULL
};
void _module_load() {
	addCommand("QUIT", m_quit, MAXPARA, MSG_NONE);
	RequiredModules.m_quit = 1;
	return;
}
void _module_unload() {
	delCommand("QUIT", m_quit);
	RequiredModules.m_quit = 0;
	return;
}
#endif

/*
 * m_quit
 *	parv[0] = sender prefix
 *	parv[1] = comment
 */
int m_quit(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char comment[TOPICLEN + 1];
	int validmsg = (parc > 1 && *parv[1] != '\0') ? 1 : 0;

	sptr->flags |= FLAGS_NORMALEX;
	if (IsServer(cptr)) {
		return exit_client(cptr, sptr, sptr, (validmsg == 1) ? parv[1] : parv[0]);
	}
	if ((!HasMode(sptr, UMODE_OPER) && (sptr->firsttime + ANTI_SPAM_QUIT_MESSAGE_TIME) > timeofday) ||
	  validmsg == 0 || CUSTOMQUITMESSAGES == 0) {
		return exit_client(cptr, sptr, sptr, "Client Quit");
	}
	else {
		if (strlen(parv[1]) > (TOPICLEN - 7)) {
			parv[1][TOPICLEN - 7] = '\0';
		}
		ircsprintf(comment, "Quit: %s", parv[1]);
		return exit_client(cptr, sptr, sptr, comment);
	}
	return 0;
}
