/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_rakill.c: RAKILL command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_rakill.c,v 1.11 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern void send_globops(char *, ...);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_rakill.so",
	"/RAKILL command",
	4,
	"$Revision: 1.11 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("RAKILL", m_rakill, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("RAKILL", m_rakill);
	return;
}
#endif

/*
 * m_rakill
 *	parv[0] = sender prefix
 *	parv[1] = host
 *	parv[2] = user
 */
int m_rakill(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *userhost;
	ConfigItem_ban *ban;
	ConfigItem c;

	if (!IsServer(sptr) || parc < 3) {
		return 0;
	}
	if (!IsULine(sptr)) {
		send_globops("Non-ULined server %s is trying to RAKILL!", parv[0]);
		sendto_serv_butone(&me, ":%s GLOBOPS :Non-ULined server %s is trying "
			"to RAKILL!", me.name, parv[0]);
		return 0;
	}

	userhost = make_user_host(parv[2], parv[1]);
	for (ban = ci_ban; ban != NULL; ban = (ConfigItem_ban *)ban->next) {
		if (ban->type & BAN_USERHOST && ban->type & BAN_USERHOST_S) {
			if (!match(ban->mask, userhost) || !match(userhost, ban->mask)) {
				ircdlog(LOG_BAN, "AKILL on %s removed by %s", ban->mask, get_client_name(sptr, TRUE));
				c.next = del_ConfigItem((ConfigItem *)ban, (ConfigItem **)&ci_ban);
				MyFree(ban->mask);
				MyFree(ban->reason);
				MyFree(ban);
				ban = (ConfigItem_ban *)&c;
			}
		}
	}
	sendto_serv_butone(cptr, ":%s RAKILL %s %s", parv[0], parv[1], parv[2]);
	return 0;
}
