/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_sakick.c: SAKICK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_sakick.c,v 1.4 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_sakick.so",
	"/SAKICK command",
	4,
	"$Revision: 1.4 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("SAKICK", m_sakick, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("SAKICK", m_sakick);
	return;
}
#endif

/*
 * m_sakick
 *	parv[0] = sender prefix
 *	parv[1] = channel
 *	parv[2] = user
 *	parv[3] = reason
 */
int m_sakick(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	aClient *acptr;
	char *comment = (parc > 3 && *parv[3] != '\0') ? parv[3] : parv[0], *p;
	int chasing = 0;

	if (!IsPrivileged(sptr) || !HasMode(sptr, UMODE_SADMIN)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);	
		return 0;
	}
	if (parc < 3) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "SAKICK");
		return 0;
	}
	if ((p = strchr(parv[1], ',')) != NULL) {
		*p = '\0';
	}
	if ((p = strchr(parv[2], ',')) != NULL) {
		*p = '\0';
	}
	if ((chptr = find_channel(parv[1], NULL)) == NULL) {
		if (MyClient(sptr)) {
			sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0], parv[1]);
		}
		return 0;
	}
	if ((acptr = find_chasing(sptr, parv[2], &chasing)) == NULL) {
		return 0;
	}
	if (!IsMember(acptr, chptr)) {
		if (MyClient(sptr)) {
			sendto_one(sptr, err_str(ERR_USERNOTINCHANNEL), me.name, parv[0], chptr->chname,
				acptr->name);
		}
		return 0;
	}
	if (strlen(comment) > TOPICLEN) {
		comment[TOPICLEN] = '\0';
	}
	sendto_serv_butone(NULL, ":%s GLOBOPS :%s used SAKICK %s %s (%s)", me.name, sptr->name, chptr->chname,
		acptr->name, comment);
	sendto_realops("%s used SAKICK %s %s (%s)", sptr->name, chptr->chname, acptr->name, comment);
	sendto_channel_butserv(chptr, sptr, ":%s KICK %s %s :%s", parv[0], chptr->chname, acptr->name, comment);
	sendto_match_servs(chptr, cptr, ":%s SAKICK %s %s :%s", parv[0], chptr->chname, acptr->name, comment);
	remove_user_from_channel(acptr, chptr);
	return 0;
}
