/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_unkline.c: UNKLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_unkline.c,v 1.16 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_unkline.so",
	"/UNKLINE command",
	4,
	"$Revision: 1.16 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("UNKLINE", m_unkline, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("UNKLINE", m_unkline);
	return;
}
#endif

/*
 * m_unkline
 *	parv[0] = sender prefix
 *	parv[1] = mask
 */
int m_unkline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *user, *host, *userhost;
	ConfigItem_ban *ban;
	ConfigItem c;
	int i = 0;

	if (!OPHasFlag(sptr, OFLAG_UNKLINE)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}
	if (parc < 2 || BadPtr(parv[1])) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "UNKLINE");
		return 0;
	}
	if ((host = strchr(parv[1], '@')) != NULL || (*parv[1] == '*' || strchr(parv[1], '.'))) {
		if (host != NULL) {
			user = parv[1];
			*host++ = '\0';
		}
		else {
			user = "*";
			host = parv[1];
		}
	}
	else {
		user = "*";
		host = parv[1];
	}

	userhost = make_user_host(user, host);
	for (ban = ci_ban; ban != NULL; ban = (ConfigItem_ban *)ban->next) {
		if (ban->type & BAN_USERHOST && !(ban->type & BAN_USERHOST_S) && ban->flag.removable == 1) {
			if (!match(ban->mask, userhost) || !match(userhost, ban->mask)) {
				ircdlog(LOG_BAN, "KLINE on %s (%s) removed by %s!%s@%s as %s",
					ban->mask, ban->reason, sptr->name, sptr->username,
					sptr->host, userhost);
				c.next = del_ConfigItem((ConfigItem *)ban, (ConfigItem **)&ci_ban);
				MyFree(ban->mask);
				MyFree(ban->reason);
				MyFree(ban);
				ban = (ConfigItem_ban *)&c;
				i++;
			}
		}
	}
	if (i > 0) {
		sendto_realops("%s (%s@%s) has removed %d K:Line%s matching %s",
			sptr->name, sptr->username, MaskedHost(sptr), i, i == 1 ? "" : "s", userhost);
	}
	else {
		sendto_one(sptr, ":%s NOTICE %s :*** %s is not currently on the K:Line list, or is not removable.",
			me.name, parv[0], userhost);
	}
	return 0;
}
