/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_unszline.c: UNSZLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_unszline.c,v 1.4 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_unszline.so",
	"/UNSZLINE command",
	4,
	"$Revision: 1.4 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("UNSZLINE", m_unszline, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("UNSZLINE", m_unszline);
	return;
}
#endif

/*
 * m_unszline
 *	parv[0] = sender prefix
 *	parv[1] = nickname
 */
int m_unszline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	ConfigItem b;
	ConfigItem_ban *ban;
	int matchit = 0;
	char *mask;

	if (!(IsServer(sptr) || IsULine(sptr))) {
		return 0;
	}
	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "UNSQLINE");
		return 0;
	}
	if (parc == 3) {
		matchit = atoi(parv[1]);
		mask = parv[2];
	}
	else {
		mask = parv[1];
	}
	for (ban = ci_ban; ban != NULL; ban = (ConfigItem_ban *)ban->next) {
		if (ban->type & BAN_IP && ban->type & BAN_IP_S) {
			if (matchit ? !match(mask, ban->mask) : !mycmp(mask, ban->mask)) {
				b.next = del_ConfigItem((ConfigItem *)ban, (ConfigItem **)&ci_ban);
				MyFree(ban->mask);
				MyFree(ban->reason);
				MyFree(ban);
				ban = (ConfigItem_ban *)&b;
			}
		}
	}
	if (parc == 3) {
		sendto_serv_butone(cptr, ":%s UNSZLINE %d :%s", sptr->name, matchit, mask);
	}
	else {
		sendto_serv_butone(cptr, ":%s UNSZLINE :%s", sptr->name, mask);
	}
	return 0;
}
