/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_watch.c: WATCH command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_watch.c,v 1.11 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

static char buf[BUFSIZE];

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_watch.so",
	"/WATCH command",
	4,
	"$Revision: 1.11 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("WATCH", m_watch, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("WATCH", m_watch);
	return;
}
#endif

static void show_watch(aClient *sptr, char *name, int rpl1, int rpl2)
{
	aClient *acptr = NULL;
	if ((acptr = find_person(name, NULL)) != NULL) {
		sendto_one(sptr, rpl_str(rpl1), me.name, sptr->name, acptr->name,
			acptr->username, MaskedHost(acptr), acptr->lasttime);
	}
	else {
		sendto_one(sptr, rpl_str(rpl2), me.name, sptr->name, name, "*", "*", 0);
	}
	return;
}

/*
 * m_watch
 *	parv[0] = sender prefix
 *	parv[1] = parameter
 */
int m_watch(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *s, *p = NULL, *user, def[2] = "l";

	if (parc < 2) {
		parc = 2;
		parv[1] = def;
	}
	for (s = strtoken(&p, parv[1], ","); s != NULL; s = strtoken(&p, NULL, ",")) {
		if ((user = strchr(s, '!')) != NULL) {
			*user++ = '\0';
		}
		if (*s == '+') {
			if (*(s + 1) != '\0') {
				if (sptr->user->watches >= MAXWATCH) {
					sendto_one(sptr, err_str(ERR_TOOMANYWATCH), me.name,
						sptr->name, s + 1);
					continue;
				}
				add_to_watch_hash_table(s + 1, sptr);
			}
			show_watch(sptr, s + 1, RPL_NOWON, RPL_NOWOFF);
			continue;
		}
		if (*s == '-') {
			del_from_watch_hash_table(s + 1, sptr);
			show_watch(sptr, s + 1, RPL_WATCHOFF, RPL_WATCHOFF);
			continue;
		}
		if (*s == 'c' || *s == 'C') {
			hash_del_watch_list(sptr);
			continue;
		}
		if (*s == 's' || *s == 'S') {
			Link *lp;
			aWatch *anptr;
			int count = 0;

			if ((anptr = hash_get_watch(sptr->name)) != NULL) {
				for (lp = anptr->watch; lp != NULL; lp = lp->next) {
					count++;
				}
			}
			sendto_one(sptr, rpl_str(RPL_WATCHSTAT), me.name, parv[0], sptr->user->watches,
				count);

			*buf = '\0';
			if ((lp = sptr->user->watch) == NULL) {
				sendto_one(sptr, rpl_str(RPL_ENDOFWATCHLIST), me.name, parv[0], *s);
				continue;
			}

			strcpy(buf, lp->value.wptr->nick);
			count = strlen(parv[0]) + strlen(me.name) + strlen(buf) + 10;
			while (lp != NULL) {
				if (count + strlen(lp->value.wptr->nick) + 1 > BUFSIZE - 2) {
					sendto_one(sptr, rpl_str(RPL_WATCHLIST), me.name, parv[0],
						buf);
					*buf = '\0';
					count = strlen(parv[0]) + strlen(me.name) + 10;
				}
				strcat(buf, " ");
				strcat(buf, lp->value.wptr->nick);
				count += strlen(lp->value.wptr->nick) + 1;
				lp = lp->next;
			}
			sendto_one(sptr, rpl_str(RPL_WATCHLIST), me.name, parv[0], buf);
			sendto_one(sptr, rpl_str(RPL_ENDOFWATCHLIST), me.name, parv[0], *s);
			continue;
		}
		if (*s == 'l' || *s == 'L') {
			Link *lp;
			for (lp = sptr->user->watch; lp != NULL; lp = lp->next) {
				if ((acptr = find_person(lp->value.wptr->nick, NULL)) != NULL) {
					sendto_one(sptr, rpl_str(RPL_NOWON), me.name, parv[0], acptr->name,
						acptr->username, MaskedHost(acptr), acptr->tsinfo);
				}
				else if (isupper(*s)) {
					sendto_one(sptr, rpl_str(RPL_NOWOFF), me.name, parv[0],
						lp->value.wptr->nick, "*", "*", lp->value.wptr->lasttime);
				}
			}
			sendto_one(sptr, rpl_str(RPL_ENDOFWATCHLIST), me.name, parv[0], *s);
			continue;
		}
	}
	return 0;
}
