/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/bsd.c: socket data delivery
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: bsd.c,v 1.21 2002/10/26 13:54:01 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "memory.h"

extern int errno;
#ifndef SYS_ERRLIST_DECLARED
extern char *sys_errlist[];
#endif
#ifdef DEBUGMODE
int writecalls = 0;
int readcalls = 0;
int writeb[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
#endif

#ifdef HAVE_OPENSSL_ON
extern int safe_SSL_write(aClient *, const void *, size_t);
extern int safe_SSL_read(aClient *, void *, size_t);
#endif

int ircsend(aClient *cptr, const void *ptr, size_t size)
{
#ifdef HAVE_OPENSSL_ON
	if (cptr->flags & FLAGS_SECURE && cptr->ssl != NULL) {
		return safe_SSL_write(cptr, ptr, size);
	}
#endif
	return send(cptr->fd, ptr, size, 0);
}

int ircrecv(aClient *cptr, void *ptr, size_t size)
{
#ifdef HAVE_OPENSSL_ON
	if (cptr->flags & FLAGS_SECURE && cptr->ssl != NULL) {
		return safe_SSL_read(cptr, ptr, size);
	}
#endif
	return recv(cptr->fd, ptr, size, 0);
}

int deliver_it(aClient *cptr, char *str, int len)
{
	int retval;

#ifdef DEBUGMODE
	writecalls++;
#endif

	retval = ircsend(cptr, str, len);
	if (retval < 0 && (errno == EWOULDBLOCK || errno == EAGAIN || errno == ENOBUFS)) {
		retval = 0;
		cptr->flags |= FLAGS_BLOCKED;
	}
	else if (retval > 0) {
		cptr->flags &= ~FLAGS_BLOCKED;
	}
#ifdef DEBUGMODE
	if (retval < 0) {
		writeb[0]++;
		Debug((DEBUG_ERROR, "Write error (%s) to %s", sys_errlist[errno], cptr->name));
	}
	else if (retval == 0) {
		writeb[1]++;
	}
	else if (retval < 16) {
		writeb[2]++;
	}
	else if (retval < 32) {
		writeb[3]++;
	}
	else if (retval < 64) {
		writeb[4]++;
	}
	else if (retval < 128) {
		writeb[5]++;
	}
	else if (retval < 256) {
		writeb[6]++;
	}
	else if (retval < 512) {
		writeb[7]++;
	}
	else if (retval < 1024) {
		writeb[8]++;
	}
	else {
		writeb[9]++;
	}
#endif
	if (retval > 0) {
		cptr->sendB += retval;
		if (cptr->sendB & 0x0400) {
			cptr->sendK += (cptr->sendB >> 10);
			cptr->sendB &= 0x03ff;
		}

		me.sendB += retval;
		if (me.sendB & 0x0400) {
			me.sendK += (me.sendB >> 10);
			me.sendB &= 0x03ff;
		}

		if (cptr->listener != NULL) {
			cptr->listener->sendB += retval;
			if (cptr->listener->sendB & 0x0400) {
				cptr->listener->sendK += (cptr->listener->sendB >> 10);
				cptr->listener->sendB &= 0x03ff;
			}
		}
	}
	return retval;
}
