/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/fdlist.c: file descriptor list management
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: fdlist.c,v 1.5 2002/10/25 22:42:23 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "config.h"
#include "fdlist.h"
#include "memory.h"

fdlist serv_fdlist;
#ifndef NO_PRIORITY
fdlist busycli_fdlist;
#endif
fdlist default_fdlist;
fdlist oper_fdlist;
fdlist listen_fdlist;

void addto_fdlist(int fd, fdlist *listp)
{
	int index;

	if ((index = ++listp->last_entry) > MAXCONNECTIONS) {
		listp->last_entry--;
		ircdlog(LOG_ERROR, "fdlist too big, exiting");
		abort();
	}
	else {
		listp->entry[index] = fd;
	}
	return;
}

void delfrom_fdlist(int fd, fdlist *listp)
{
	int i;

	for (i = listp->last_entry; i; i--) {
		if (listp->entry[i] == fd) {
			break;
		}
	}
	if (i == 0) {
		return;
	}
	if (i == listp->last_entry) {
		listp->entry[i] = 0;
		listp->last_entry--;
	}
	else {
		listp->entry[i] = listp->entry[listp->last_entry];
		listp->entry[listp->last_entry] = 0;
		listp->last_entry--;
	}
	return;
}

void init_fdlist(fdlist *listp)
{
	listp->last_entry = 0;
	memset((char *)listp->entry, '\0', sizeof(listp->entry));
	return;
}

#ifndef NO_PRIORITY

#ifdef CLIENT_SERVER
#define BUSY_CLIENT(x) (((x)->priority < 55) || (!lifesux && ((x)->priority < 75)))
#else
#define BUSY_CLIENT(x) (((x)->priority < 40) || (!lifesux && ((x)->priority < 60)))
#endif

void check_fdlists()
{
	aClient *cptr;
	int i, j = 0;

	for (i = highest_fd; i > 0; i--) {
		if (!(cptr = local[i])) {
			continue;
		}
		if (IsServer(cptr) || HasMode(cptr, UMODE_OPER)) {
			busycli_fdlist.entry[++j] = i;
			continue;
		}
		if (cptr->receiveM == cptr->lastrecvM) {
			if ((cptr->priority += 2) > 90) {
				cptr->priority = 90;
			}
			else if (BUSY_CLIENT(cptr)) {
				busycli_fdlist.entry[++j] = i;
			}
			continue;
		}
		else {
			cptr->lastrecvM = cptr->receiveM;
			if ((cptr->priority -= 30) < 0) {
				cptr->priority = 0;
				busycli_fdlist.entry[++j] = i;
			}
			else if (BUSY_CLIENT(cptr)) {
				busycli_fdlist.entry[++j] = i;
			}
		}
	}
	busycli_fdlist.last_entry = j;
}
#endif

void flush_fdlist_connections(fdlist *listp)
{
	int i, fd;
	aClient *cptr;

	for (fd = listp->entry[i = 1]; i <= listp->last_entry; fd = listp->entry[++i]) {
		if (!(cptr = local[fd]) || cptr->flags & FLAGS_BLOCKED || DBufLength(&cptr->sendQ) <= 0) {
			continue;
		}
		send_queued(cptr);
	}
	return;
}
