/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/memory.c: internal memory management
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: memory.c,v 1.2 2002/06/09 18:47:06 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "numeric.h"
#include "blalloc.h"
#include "setup.h"
#include "memory.h"
#include <stdlib.h>
#include <string.h>

extern void *BlockHeapALLOC(BlockHeap *);
extern int BlockHeapFREE(BlockHeap *, void *);

void *_MyMalloc(size_t size, char *file, int line)
{
	void *m = malloc(size);
	if (m == NULL) {
		outofmemory();
	}
	return m;
}

void *_MyMallocX(size_t size, char *file, int line)
{
	void *m = malloc(size);
	if (m == NULL) {
		outofmemory();
	}
	memset(m, '\0', size);
	return m;
}

void *_MyRealloc(void *p, size_t size, char *file, int line)
{
	void *ret;

	if (p == NULL) {
		return _MyMalloc(size, file, line);
	}
	if (size == 0) {
		_MyFree(p, file, line);
		return NULL;
	}

	ret = realloc(p, size);
	if (ret == NULL) {
		outofmemory();
	}
	return ret;
}

void _MyFree(void *p, char *file, int line)
{
	if (p != NULL) {
		free(p);
	}
	return;
}

void _DupString(char **p, const char *s, char *file, int line)
{
	*p = _MyMalloc(strlen(s) + 1, file, line);
	strcpy(*p, s);
	return;
}

void *_BlockHeapAlloc(BlockHeap *bh, char *file, int line)
{
#ifdef HAVE_BLALLOC_ON
	void *ret = BlockHeapALLOC(bh);
	if (ret == NULL) {
		outofmemory();
	}
	return ret;
#else
	return _MyMalloc((size_t)bh, file, line);
#endif
}

int _BlockHeapFree(BlockHeap *bh, void *p, char *file, int line)
{
#ifdef HAVE_BLALLOC_ON
	if (bh != NULL && p != NULL) {
		return BlockHeapFREE(bh, p);
	}
#else
	if (p != NULL) {
		_MyFree(p, file, line);
	}
#endif
	return 0;
}

void outofmemory(void)
{
	static int was_here = 0;

	if (was_here == 1) {
		abort();
	}
	was_here = 1;

	ircdlog(LOG_ERROR, "FATAL: out of memory, restarting server.");
	Debug((DEBUG_FATAL, "Out of memory: restarting server..."));
	restart("Out of memory");
}
