/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/packet.c: internet packet handling
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: packet.c,v 1.15 2002/11/20 22:26:36 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "msg.h"
#include "h.h"
#include "ssl.h"
#include "zlink.h"
#include "memory.h"

int dopacket(aClient *cptr, char *buffer, int length)
{
	char *ch1, *ch2, *cptrbuf = cptr->buffer, *nbuf = NULL;
	int nlen;

#ifdef HAVE_OPENSSL_ON
	if (cptr->serv->sflags & SFLAGS_RC4_IN) {
		rc4_process_stream(cptr->serv->rc4_in, buffer, length);
	}
#endif

	cptr->receiveB += length;
	if (cptr->receiveB & 0x0400) {
		cptr->receiveK += (cptr->receiveB >> 10);
		cptr->receiveB &= 0x03ff;
	}

	me.receiveB += length;
	if (me.receiveB & 0x0400) {
		me.receiveK += (me.receiveB >> 10);
		me.receiveB &= 0x03ff;
	}
	if (cptr->listener != NULL) {
		cptr->listener->receiveB += length;
		if (cptr->listener->receiveB & 0x0400) {
			cptr->listener->receiveK += (cptr->listener->receiveB >> 10);
			cptr->listener->receiveB &= 0x03ff;
		}
	}

		zcontinue:
	ch1 = cptrbuf + cptr->count;
	ch2 = buffer;

	if (cptr->serv->sflags & SFLAGS_ZIP_IN) {
		int err;
		ch2 = zip_input(cptr->serv->zip_in, ch2, &length, &err, &nbuf, &nlen);
		if (length == -1) {
			sendto_realops("ZipIn error for %s: (%d) %s", cptr->name, err, ch2);
			return exit_client(cptr, cptr, &me, "Fatal error in zip_input()");
		}
	}
	while (--length >= 0) {
		char g = (*ch1 = *ch2++);
		if (g < '\16' && (g == '\n' || g == '\r')) {
			if (ch1 == cptrbuf) {
				continue;
			}

			*ch1 = '\0';
			me.receiveM++;
			cptr->receiveM++;
			if (cptr->listener != NULL) {
				cptr->listener->receiveM++;
			}
			cptr->count = 0;

			switch (parse(cptr, cptr->buffer, ch1)) {
				case FLUSH_BUFFER:
					return FLUSH_BUFFER;
				case ZIP_NEXT_BUFFER:
					if (length) {
						int err;
						ch2 = zip_input(cptr->serv->zip_in, ch2, &length, &err, &nbuf, &nlen);
						if (length == -1) {
							sendto_realops("ZipIn error for %s: (%d) %s", cptr->name,
								err, ch2);
							return exit_client(cptr, cptr, &me, "Fatal error in zip_input()");
						}
					}
					break;
#ifdef HAVE_OPENSSL_ON
				case RC4_NEXT_BUFFER:
					if (length) {
						rc4_process_stream(cptr->serv->rc4_in, ch2, length);
					}
					break;
#endif
				default:
					break;
			}

			if (cptr->flags & FLAGS_DEADSOCKET) {
				return exit_client(cptr, cptr, &me,
					cptr->flags & FLAGS_SENDQEX ? "SendQ Exceeded" : "Dead Socket");
			}

			ch1 = cptrbuf;
		}
		else if (ch1 < cptrbuf + (sizeof(cptr->buffer) - 1)) {
			ch1++;
		}
	}

	cptr->count = ch1 - cptrbuf;
	if (nbuf != NULL) {
		buffer = nbuf;
		length = nlen;
		nbuf = NULL;
		goto zcontinue;
	}
	return 0;
}

int client_dopacket(aClient *cptr, char *buffer, int length)
{
	strncpy(cptr->buffer, buffer, BUFSIZE);
	length = strlen(cptr->buffer);

	++me.receiveM;
	++cptr->receiveM;

	cptr->receiveB += length;
	if (cptr->receiveB > 1023) {
		cptr->receiveK += cptr->receiveB >> 10;
		cptr->receiveB &= 0x03ff;
	}

	me.receiveB += length;
	if (me.receiveB > 1023) {
		me.receiveK += me.receiveB >> 10;
		me.receiveB &= 0x03ff;
	}

	cptr->count = 0;

	if (FLUSH_BUFFER == parse(cptr, cptr->buffer, cptr->buffer + length)) {
		return FLUSH_BUFFER;
	}
	else if (cptr->flags & FLAGS_DEADSOCKET) {
		return exit_client(cptr, cptr, &me,
			cptr->flags & FLAGS_SENDQEX ? "SendQ Exceeded" : "Dead Socket");
	}
	return 1;
}
