/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/rc4.c: rc4 link encryption
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: rc4.c,v 1.7 2002/10/24 19:28:14 rageircd Exp $
 */

#include "setup.h"
#include "ssl.h"
#include "memory.h"

#ifdef HAVE_OPENSSL_ON

void *rc4_initstate(unsigned char *key, int keylen)
{
	RC4_word i;
	RC4_byte tmp, idx1 = 0, idx2 = 0;
	RC4_state *rc4;

	if ((sizeof(RC4_byte) != 1) || (sizeof(RC4_word) != 4)) {
		abort();
	}

	rc4 = (RC4_state *)MyMallocX(sizeof(RC4_state));
	for (i = 0; i < 256; i++) {
		rc4->mstate[i] = (RC4_byte)i;
	}
	for (i = 0; i < 256; i++) {
		idx2 = (key[idx1++] + rc4->mstate[i] + idx2);
		if (idx2 > 255) {
			abort();
		}

		tmp = rc4->mstate[i];
		rc4->mstate[i] = rc4->mstate[idx2];
		rc4->mstate[idx2] = tmp;

		if (idx1 >= keylen) {
			idx1 = 0;
		}
	}
	return (void *)rc4;
}

void rc4_process_stream(void *rc4_context, unsigned char *istring, unsigned int stringlen)
{
	RC4_state *rc4 = (RC4_state *)rc4_context;
	RC4_byte *s = rc4->mstate;
	RC4_word x = rc4->x, y = rc4->y;

	while (stringlen--) {
		RC4_word a, b;

		x = (x + 1) & 0xFF;
		a = s[x];
		y = (y + a) & 0xFF;
		b = s[y];
		s[x] = b;
		s[y] = a;
		*istring++ ^= s[(a + b) & 0xFF];
	}
	rc4->x = (RC4_byte)x;
	rc4->y = (RC4_byte)y;
	return;
}

void rc4_process_stream_to_buf(void *rc4_context, const unsigned char *istring, unsigned char *ostring,
	unsigned int stringlen)
{
	RC4_state *rc4 = (RC4_state *)rc4_context;
	RC4_byte *s = rc4->mstate;
	RC4_word x = rc4->x, y = rc4->y;

	while (stringlen--) {
		RC4_word a, b;

		x = (x + 1) & 0xFF;
		a = s[x];
		y = (y + a) & 0xFF;
		b = s[y];
		s[x] = b;
		s[y] = a;
		*ostring++ = *istring++ ^ s[(a + b) & 0xFF];
	}
	rc4->x = (RC4_byte)x;
	rc4->y = (RC4_byte)y;
	return;
}

void rc4_destroystate(void *rc4_context)
{
	memset(rc4_context, '\0', sizeof(RC4_state));
	free(rc4_context);
	return;
}

#endif /* HAVE_OPENSSL_ON */
